/* $Id: linuxdd.c,v 1.2 2005/12/18 20:57:58 root Exp $ */

/* Linux physical driver framework */

#include <linux/autoconf.h>

#include <linux/version.h>
#define _LOOSE_KERNEL_NAMES
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/timer.h>
#include <linux/file.h>

#include <linux/config.h>
#include <linux/mm.h>
#include <linux/kernel_stat.h>
#include <linux/sched.h>
#include <linux/version.h>
#include <linux/fs.h>
#include <asm/uaccess.h>

#include <quietpc.h> 
#include <quietpcp.h>
#include <hwmon.h>
#define RC_BUG                     1

#define TIMER_GRANULARITY    (HZ/10)

MODULE_DESCRIPTION("Smart fan control");
MODULE_AUTHOR("Andrew Belov");
MODULE_LICENSE("public-domain");

static void linux_tick_handler(unsigned long dummy);

/* Event interface */

struct timer_list tl=TIMER_INITIALIZER(linux_tick_handler, 0, 0);

/* The Linux Tick Handler */

static void linux_tick_handler(unsigned long dummy)
{
 tick_handler();
 del_timer(&tl);
 tl.expires=jiffies+TIMER_GRANULARITY;
 add_timer(&tl);
}

/* Installs the handler */

int install_tick_handler()
{
 tl.expires=jiffies+TIMER_GRANULARITY;
 add_timer(&tl);
 return(0);
}

/*
 * Device communication functions
 */

/* open() */

static int hwmon_open(struct inode *inode, struct file *file)
{
 return(0);
}

/* close() */

static int hwmon_release(struct inode *inode, struct file *file)
{
 return(0);
}

/* ioctl() */

static int hwmon_ioctl_router(struct inode *inode, struct file *file,
                              unsigned int func, unsigned long param)
{
 char xferbuf[1024];
 int rc;
 struct ioctl_encaps prm;
 void *uptr, *pptr, *dptr;

 uptr=(void *)param;
 copy_from_user(&prm, uptr, sizeof(prm));
 if(prm.parm_len+prm.data_len>sizeof(xferbuf))
  return(RC_BUG);
 pptr=xferbuf;
 dptr=pptr+prm.parm_len;
 if(prm.parm_len>0)
  copy_from_user(pptr, prm.pparm, prm.parm_len);
 if(prm.data_len>0)
  copy_from_user(dptr, prm.pdata, prm.data_len);
 rc=hwmon_ioctl(func, pptr, prm.parm_len, dptr, prm.data_len);
 if(prm.data_len>0)
  copy_to_user(prm.pdata, dptr, prm.data_len);
 return(rc);
}

/* File operations */

struct file_operations quietpc_fops=
{
 .owner = THIS_MODULE,
 .ioctl = hwmon_ioctl_router,
 .open = hwmon_open,
 .release = hwmon_release,
};

/* Startup */

int init_module()
{
 int rc;

 if((rc=configure_hwmon()))
 {
  printk("QuietPC: failed to configure hardware, rc=%d\n", rc);
  return(rc);
 }
 if((rc=register_chrdev(MAJOR_NUM, DEVICE_NAME, &quietpc_fops))<0)
 {
  printk("QuietPC: character device creation failed, rc=%d\n", rc);
  shutdown_hwmon();
  return(rc);
 }
 return(0);
}

/* Shutdown */

void cleanup_module()
{
 unregister_chrdev(MAJOR_NUM, DEVICE_NAME);
 shutdown_hwmon();
 del_timer(&tl);
}
